import pandas as pd
import time
import os
import psutil
import gc
import sys

class rasterFuzzyTSC:

    def __init__(self, trainingFile, testFile):
        self.startTime = time.time()
        self.training = pd.read_csv(trainingFile, sep='\t', header=None)  # np.loadtxt(sys.argv[1], delimiter='\t')
        self.testing = pd.read_csv(testFile, sep='\t', header=None)  # np.loadtxt(sys.argv[2],  delimiter='\t')
        #self.training = np.loadtxt(trainingFile, delimiter='\t')
        #self.testing = np.loadtxt(testFile,  delimiter='\t')
        # To drop the first column which is a class label of each sample
        self.training.drop(0, axis='columns', inplace=True)

    def Run(self):

        # separating classes into different dictionary variables and creating mean, min and max curves
        maxData = self.training.max(axis=0)
        meanData = self.training.mean(axis=0)
        minData = self.training.min(axis=0)

        del self.training
        gc.collect()

        # Classification Phase

        counter = {}
        num_rows, num_columns = self.testing.shape

        # print(num_rows, num_columns)
        newColumn = list()
        # print(num_columns)
        correct = 0
        for i in range(num_rows):
            counter = 0.0
            for j in range(1, num_columns):
                if self.testing.iloc[i, j] >= meanData[j]:
                    if self.testing.iloc[i, j] <= maxData[j]:
                        counter = counter + 0.5 * (self.testing.iloc[i, j] - meanData[j]) / (maxData[j] - meanData[j])
                    else:
                        counter = counter + 1
                else:
                    if self.testing.iloc[i, j] >= minData[j]:
                        counter = counter + 0.5 * (meanData[j] - self.testing.iloc[i, j]) / (meanData[j] - minData[j])
                    else:
                        counter = counter + 1

            counter = counter / num_columns

            newColumn.append(counter)

        self.testing['RD'] = newColumn
        print("Final test samples after calculating the Ravi Distance (RD):")
        print(self.testing)

        N = int(input("Enter the how many number of top elements to be retrieved between: 1 to " + str(num_rows) + ":"))
        sortedDF = self.testing.sort_values('RD').head(N)
        print("Final top- " + str(N) + " testing samples retrieved based on the Ravi Distance:")
        print(sortedDF)

        # Statistics of the Algorithm after execution
        print("Dataset name:", sys.argv[1])
        print("Total Execution time of proposedAlgo", time.time() - self.startTime)
        process = psutil.Process(os.getpid())
        memory = process.memory_full_info().uss
        memory_in_KB = memory / 1024
        print("Memory of proposedAlgo in KB:", memory_in_KB)  # in bytes


if __name__ == '__main__':
    obj = rasterFuzzyTSC(sys.argv[1], sys.argv[2])
    obj.Run()




import numpy as np
import pandas as pd
import time
import math
import sys
import os
import psutil

class rasterOneNNHausdorff:

    def __init__(self, trainingFile, testFile):
        self.training = pd.read_csv(sys.argv[1], sep='\t', header=None)  # np.loadtxt(sys.argv[1], delimiter='\t')
        self.testing = pd.read_csv(sys.argv[2], sep='\t', header=None)  # np.loadtxt(sys.argv[2],  delimiter='\t')

        # To drop the first column which is a class label of each sample
        self.training.drop(0, axis='columns', inplace=True)

    def hausdorff(self, u, v):
        row, = u.shape
        lea_distance = 0
        for i in range(row):
            distance1 = np.amin(np.absolute((np.diff(u[i] - v))))
            if distance1 > lea_distance:
                lea_distance = distance1
        return lea_distance

    def Run(self):
        num_rows_test, num_columns_test = self.testing.shape
        num_rows_train, num_columns_train = self.training.shape
        training_noclass = self.training#self.training[:, 1:]
        testing_noclass = self.testing#self.testing[:, 1:]
        #predicted_label = None
        #correct = 0
        newColumn = list()
        start_time = time.time()
        for i in range(num_rows_test):
            least_distance = float('inf')
            for j in range(num_rows_train):
                dist = max(self.hausdorff(testing_noclass.iloc[i].to_numpy(), training_noclass.iloc[j].to_numpy()),
                           self.hausdorff(training_noclass.iloc[j].to_numpy(), testing_noclass.iloc[i].to_numpy()))  # math.sqrt(squaring)
                if dist < least_distance:
                    #predicted_label = self.training[j][0]
                    least_distance = dist
            #if predicted_label == self.testing[i][0]:
            #    correct = correct + 1
            newColumn.append(least_distance)
        print(newColumn)
        self.testing['1NNmaxNorm'] = newColumn
        print("Final test samples after calculating the Ravi Distance (RD):")
        print(self.testing)

        N = int(input(
            "Enter the how many number of top elements to be retrieved between: 1 to " + str(num_rows_test) + ":"))
        sortedDF = self.testing.sort_values('1NNmaxNorm').head(N)
        print("Final top- " + str(N) + " testing samples retrieved based on the Ravi Distance:")
        print(sortedDF)
        ##accuracy = (correct / num_rows_test) * 100
        #print("Total Accuracy of oneNNHausdorff is:", accuracy)
        print("Total Execution time of oneNNHausdorff", time.time() - start_time)
        process = psutil.Process(os.getpid())
        memory = process.memory_full_info().uss
        memory_in_KB = memory / (1024)
        print("Total Memory of oneNNHausdorff inKB", memory_in_KB)  # in bytes


if __name__ == '__main__':
    obj = rasterOneNNHausdorff(sys.argv[1], sys.argv[2])
    obj.Run()


import numpy as np
import pandas as pd
import os
import psutil
import time
import sys

class rasterOneNNDTW:

    def __init__(self, trainingFile, testFile):
        self.training = pd.read_csv(sys.argv[1], sep='\t', header=None)  # np.loadtxt(sys.argv[1], delimiter='\t')
        self.testing = pd.read_csv(sys.argv[2], sep='\t', header=None)  # np.loadtxt(sys.argv[2],  delimiter='\t')

        # To drop the first column which is a class label of each sample
        self.training.drop(0, axis='columns', inplace=True)

    def dtw(self, A, B):
        N = len(A)
        M = len(B)
        d = np.zeros((N, M))
        for n in range(N):
            for m in range(M):
                d[n][m] = (A[n] - B[m]) ** 2
        D = np.zeros(d.shape)
        D[0][0] = d[0][0]
        for n in range(1, N):
            D[n][0] = d[n][0] + D[n - 1][0]
        for m in range(1, M):
            D[0][m] = d[0][m] + D[0][m - 1]
        for n in range(1, N):
            for m in range(1, M):
                D[n][m] = d[n][m] + min(D[n - 1][m], D[n - 1][m - 1], D[n][m - 1])
        return D[N - 1][M - 1]

    def Run(self):
        num_rows_test, num_columns_test = self.testing.shape
        num_rows_train, num_columns_train = self.training.shape
        training_noclass = self.training#self.training[:, 1:]
        testing_noclass = self.testing#self.testing[:, 1:]

        newColumn = list()
        #predicted_label = None
        correct = 0
        startTime = time.time()
        for i in range(num_rows_test):
            # print(i)
            least_distance = float('inf')
            for j in range(num_rows_train):
                dist = self.dtw(testing_noclass.iloc[i].to_numpy(), training_noclass.iloc[j].to_numpy())
                if dist < least_distance:
                    #predicted_label = self.training[j][0]
                    least_distance = dist

            newColumn.append(least_distance)
            #if predicted_label == self.testing[i][0]:
            #    correct = correct + 1
        print(newColumn)
        self.testing['1NNmaxNorm'] = newColumn
        print("Final test samples after calculating the Ravi Distance (RD):")
        print(self.testing)

        N = int(input("Enter the how many number of top elements to be retrieved between: 1 to " + str(num_rows_test) + ":"))
        sortedDF = self.testing.sort_values('1NNmaxNorm').head(N)
        print("Final top- " + str(N) + " testing samples retrieved based on the Ravi Distance:")
        print(sortedDF)
        #accuracy = (correct / num_rows_test) * 100
        #print("Total Accuracy of oneNNDTW is:", accuracy)
        print("Total Execution time oneNNDTW is:", time.time() - startTime)
        process = psutil.Process(os.getpid())
        memory = process.memory_full_info().uss
        memory_in_KB = memory / (1024)
        print("Total Memory of oneNNDTW inKB", memory_in_KB)  # in bytes


if __name__ == '__main__':
    obj = rasterOneNNDTW(sys.argv[1], sys.argv[2])
    obj.Run()




from sklearn.cluster import KMeans
from sklearn.cluster import DBSCAN
import psycopg2
import numpy as np
import pandas as pd

def kMeans(dataframe, k, max_iter=100):
    """
    K-Means clustering algorithm
    :param dataframe: data to be clustered
    :param k: number of clusters
    :param max_iter: maximum number of iterations
    :return: list of clusters
    """
    # initialize k-means algorithm
    data = dataframe.drop(['x', 'y'], axis=1)
    data = data.to_numpy()
    kmeans = KMeans(n_clusters=k, max_iter=max_iter).fit(data)
    # return the clusters
    #combine x ,y and labels as dataframe
    label = dataframe[['x', 'y']]
    labels = label.assign(labels=kmeans.labels_)

    return labels, kmeans.cluster_centers_

def kMeansPP(dataframe, k, max_iter=300):
    """
    K-Means++ clustering algorithm
    :param dataframe: data to be clustered
    :param k: number of clusters
    :param max_iter: maximum number of iterations
    :return: list of clusters
    """
    # initialize k-means++ algorithm
    data = dataframe.drop(['x', 'y'], axis=1)
    data = data.to_numpy()
    kmeans = KMeans(n_clusters=k, max_iter=max_iter, init='k-means++').fit(data)
    # return the clusters
    #combine x ,y and labels as dataframe
    label = dataframe[['x', 'y']]
    labels = label.assign(labels=kmeans.labels_)

    return labels, kmeans.cluster_centers_

def DBScan(dataframe, ep, min_sample):
    # initialize k-means++ algorithm
    data = dataframe.drop(['x', 'y'], axis=1)
    data = data.to_numpy()
    dbs = DBSCAN(eps=ep, min_samples=min_sample).fit(data)
    # return the clusters
    #combine x ,y and labels as dataframe
    label = dataframe[['x', 'y']]
    labels = label.assign(labels=dbs.labels_)

    return labels


#import numpy as np
import pandas as pd
#import math
import time
import psutil
import os
import sys

class rasterOneNNMaxNorm:

    def __init__(self, trainingFile, testFile):
        self.startTime = time.time()
        self.training = pd.read_csv(sys.argv[1], sep='\t', header=None)  # np.loadtxt(sys.argv[1], delimiter='\t')
        self.testing = pd.read_csv(sys.argv[2], sep='\t', header=None)  # np.loadtxt(sys.argv[2],  delimiter='\t')

        # To drop the first column which is a class label of each sample
        self.training.drop(0, axis='columns', inplace=True)

    def Run(self):
        num_rows_test, num_columns_test = self.testing.shape
        num_rows_train, num_columns_train = self.training.shape
        newColumn = list()

        self.startTime = time.time()
        for i in range(num_rows_test):
            least_distance = float('inf')
            for j in range(num_rows_train):
                maximum = float('-inf')
                for k in range(num_columns_train):
                    temp = pd.Series.abs(self.testing.iloc[i, k] - self.training.iloc[j, k])
                    #print(temp)
                    if temp > maximum:
                        maximum = temp
                dist = maximum  # math.sqrt(squaring)
                if dist < least_distance:
                    #predicted_label = self.training[j][0]
                    least_distance = dist
            newColumn.append(least_distance)
            # if predicted_label == self.testing[i][0]:
            #    correct = correct + 1

        print(newColumn)
        self.testing['1NNmaxNorm'] = newColumn
        print("Final test samples after calculating the Ravi Distance (RD):")
        print(self.testing)

        N = int(input("Enter the how many number of top elements to be retrieved between: 1 to " + str(num_rows_test) + ":"))
        sortedDF = self.testing.sort_values('1NNmaxNorm').head(N)
        print("Final top- " + str(N) + " testing samples retrieved based on the Ravi Distance:")
        print(sortedDF)
        #accuracy = (correct / num_rows_test) * 100
        #print("Total Accuracy of 1NNMaxNorm is:", accuracy)
        print("Total Execution time 1NNMaxNorm is:", time.time() - self.startTime)
        process = psutil.Process(os.getpid())
        memory = process.memory_full_info().uss
        memory_in_KB = memory / (1024)
        print("Memory of 1NNmaxinorm in KB", memory_in_KB)  # in bytes


if __name__ == '__main__':
    obj = rasterOneNNMaxNorm(sys.argv[1], sys.argv[2])
    obj.Run()